@@ -57,24 +57,31 @@ enum
 typedef struct
 {
   GtkWidget    *widget;
+  Window        window;
   gint          x;
   gint          y;
+  gint          stack_index;
+  gulong        realize_handler;
 #ifdef HAVE_X_COMPOSITE
   GtkAllocation old_allocation;
-  Window        window;
   Damage        damage;
   Picture       picture;
   Picture       alpha_mask;
   Picture       alpha_mask_unscaled;
   gint          background_width, background_height;
   gint          state;
-  gulong        realize_handler,
-                style_set_handler,
+  guint         style_set_handler,
                 size_allocate_handler,
                 state_change_handler;
 #endif
 } ChildData;
 
+typedef struct
+{
+  GdkRectangle  rect;
+  gint          stack_index;
+} Layout;
+
 typedef struct HildonHomeAreaPriv_
 {
   gboolean      layout_mode;
@@ -170,6 +177,11 @@ static void
 hildon_home_area_size_allocate (GtkWidget      *widget,
                                 GtkAllocation  *allocation);
 
+static void
+hildon_home_area_child_realize (GtkWidget             *child,
+                                HildonHomeArea        *area);
+
+
 #ifdef HAVE_X_COMPOSITE
 
 static void
@@ -204,10 +216,6 @@ static void
 hildon_home_area_child_build_alpha_mask (HildonHomeArea       *area,
                                          GtkWidget            *child);
 
-static void
-hildon_home_area_child_realize (GtkWidget             *child,
-                                HildonHomeArea        *area);
-
 static void
 hildon_home_area_child_state_change (ChildData         *child_data);
 
@@ -241,6 +249,13 @@ child_data_free (ChildData *child_data)
 
 }
 
+static gint
+find_by_window (ChildData *data, Window *w)
+{
+  return !(data->window == *w);
+}
+
+
 GType
 hildon_home_area_get_type (void)
 {
@@ -915,27 +930,30 @@ hildon_home_area_add (GtkContainer *area, GtkWidget *applet)
 
   if (HILDON_DESKTOP_IS_HOME_ITEM (applet))
     {
-      GdkRectangle *rect = NULL;
+      Layout   *layout = NULL;
 
       if (priv->layout)
         {
           const gchar *name =
               hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (applet));
 
-          rect = g_hash_table_lookup (priv->layout, name);
+          layout = g_hash_table_lookup (priv->layout, name);
         }
 
-      if (rect)
+      if (layout)
         {
           hildon_home_area_put (HILDON_HOME_AREA (area),
                                 applet,
-                                rect->x, rect->y);
-          if (rect->width > 0 && rect->height > 0)
-            gtk_widget_set_size_request (applet, rect->width, rect->height);
+                                layout->rect.x, layout->rect.y,
+                                layout->stack_index);
+          if (layout->rect.width > 0 && layout->rect.height > 0)
+            gtk_widget_set_size_request (applet,
+                                         layout->rect.width,
+                                         layout->rect.height);
         }
       else
         {
-          hildon_home_area_put (HILDON_HOME_AREA (area), applet, 0, 0);
+          hildon_home_area_put (HILDON_HOME_AREA (area), applet, 0, 0, G_MAXINT);
         }
 
       g_signal_emit_by_name (area, "layout-changed");
@@ -995,7 +1013,6 @@ hildon_home_area_child_compose (ChildData *child_data,
                     alloc->y - priv->y_offset,
                     alloc->width,
                     alloc->height);
-
 }
 
 static void
@@ -1013,7 +1030,8 @@ hildon_home_area_child_style_set (GtkWidget            *child,
 
   /* HACK: Pretend the child shaped so GDK does not add the
    * child to the parent's clip list (see #412882) */
-  ((GdkWindowObject *)child->window)->shaped = TRUE;
+  if (GTK_WIDGET_REALIZED (child))
+    ((GdkWindowObject *)child->window)->shaped = TRUE;
 
   gtk_container_child_get (GTK_CONTAINER (area), child,
                            "child-data", &child_data,
@@ -1300,19 +1318,11 @@ hildon_home_area_child_state_change (ChildData *child_data)
                 "state", &state,
                 NULL);
 
-  g_debug ("state changed to %i", state);
-
   child_data->state = state;
 
 }
 
 
-static gint
-find_by_window (ChildData *data, Window *w)
-{
-  return !(data->window == *w);
-}
-
 static gboolean
 hildon_home_area_expose (GtkWidget *widget,
                          GdkEventExpose *event)
@@ -1321,6 +1331,14 @@ hildon_home_area_expose (GtkWidget *widget,
 
   priv = HILDON_HOME_AREA_GET_PRIVATE (widget);
 
+#if 0
+  g_debug ("Got expose event on %i,%i %ix%i",
+           event->area.x,
+           event->area.y,
+           event->area.width,
+           event->area.height);
+#endif
+
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       XRectangle                rectangle;
@@ -1345,11 +1363,13 @@ hildon_home_area_expose (GtkWidget *widget,
                                    &rectangle,
                                    1);
 
+#if 0
       XFixesSetPictureClipRegion (GDK_DISPLAY (),
                                   priv->picture,
                                   0,
                                   0,
                                   region);
+#endif
 
       gdk_error_trap_push ();
       XQueryTree (GDK_DISPLAY (),
@@ -1397,27 +1417,50 @@ hildon_home_area_child_window_filter (GdkXEvent        *xevent,
                                       GdkEvent         *event,
                                       GtkWidget        *child)
 {
-  XAnyEvent            *aevent = xevent;
+  XEvent               *aevent = xevent;
   HildonHomeArea       *area = HILDON_HOME_AREA (child->parent);
   HildonHomeAreaClass  *klass = HILDON_HOME_AREA_GET_CLASS (area);
 
-
   if (aevent->type == klass->xdamage_event_base + XDamageNotify)
     {
-      XDamageNotifyEvent *ev = xevent;
-      GdkRectangle rect;
-      XserverRegion parts;
-
-      rect.x = ev->area.x + child->allocation.x;
-      rect.y = ev->area.y + child->allocation.y;
-      rect.width = ev->area.width;
-      rect.height = ev->area.height;
+      XDamageNotifyEvent       *ev = xevent;
+      XserverRegion             parts;
+      XRectangle               *rects;
+      guint                     i, n_rect;
 
       parts = XFixesCreateRegion (GDK_DISPLAY (), 0, 0);
       XDamageSubtract (GDK_DISPLAY (), ev->damage, None, parts);
-      XFixesDestroyRegion (GDK_DISPLAY (), parts);
 
-      gdk_window_invalidate_rect (child->parent->window, &rect, FALSE);
+      rects = XFixesFetchRegion (GDK_DISPLAY (),
+                                 parts,
+                                 &n_rect);
+
+      XFixesDestroyRegion (GDK_DISPLAY (),
+                           parts);
+
+      for (i = 0; i < n_rect; i++)
+        {
+          GdkRectangle  rect;
+
+          rect.x = child->allocation.x + rects[i].x;
+          rect.y = child->allocation.y + rects[i].y;
+          rect.width  = rects[i].width;
+          rect.height = rects[i].height;
+
+#if 0
+          g_debug ("Got damage on %i, %i %ix%i",
+                   rect.x,
+                   rect.y,
+                   rect.width,
+                   rect.height);
+#endif
+
+          gdk_window_invalidate_rect (GTK_WIDGET (area)->window,
+                                      &rect,
+                                      FALSE);
+        }
+
+      XFree (rects);
     }
 
   return GDK_FILTER_CONTINUE;
@@ -1470,6 +1513,33 @@ hildon_home_area_create_child_data (HildonHomeArea     *area,
                              child_data->widget);
     }
 }
+#endif
+
+static gint
+sort_by_stack_index (ChildData *child1, ChildData *child2)
+{
+  return child1->stack_index - child2->stack_index;
+}
+
+
+static void
+hildon_home_area_sort_stack (HildonHomeArea *area)
+{
+  HildonHomeAreaPriv   *priv;
+  GList                *child;
+
+  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
+
+  priv->children_data = g_list_sort (priv->children_data,
+                                     (GCompareFunc)sort_by_stack_index);
+
+  for (child = priv->children_data; child ; child = child->next)
+    {
+      ChildData        *data = child->data;
+      if (GTK_WIDGET_REALIZED (data->widget))
+        gdk_window_raise (data->widget->window);
+    }
+}
 
 static void
 hildon_home_area_child_realize (GtkWidget             *child,
@@ -1481,6 +1551,9 @@ hildon_home_area_child_realize (GtkWidget             *child,
                            "child-data", &data,
                            NULL);
 
+  data->window = GDK_WINDOW_XID (child->window);
+#ifdef HAVE_X_COMPOSITE
+
   /* HACK: Pretend the child shaped so GDK does not add the
    * child to the parent's clip list (see #412882) */
   ((GdkWindowObject *)child->window)->shaped = TRUE;
@@ -1490,9 +1563,11 @@ hildon_home_area_child_realize (GtkWidget             *child,
 
   if (data->alpha_mask == None)
     hildon_home_area_child_build_alpha_mask (area, child);
-}
 #endif
 
+  hildon_home_area_sort_stack (area);
+}
+
 static GdkRectangle *
 create_rectangle (gint x, gint y, gint w, gint h)
 {
@@ -1709,16 +1784,19 @@ hildon_home_area_batch_add (HildonHomeArea *area)
           if (r->width  >= req.width &&
               r->height >= req.height)
             {
-              GdkRectangle     *layout = create_rectangle (r->x,
-                                                           r->y,
-                                                           req.width,
-                                                           req.height);
+              Layout           *layout = g_new (Layout, 1);
               GdkRectangle     *padded_layout = g_new (GdkRectangle, 1);
 
+              layout->rect.x = r->x;
+              layout->rect.y = r->y;
+              layout->rect.width = req.width;
+              layout->rect.height = req.height;
+              layout->stack_index = G_MAXINT;
+
               g_hash_table_insert (priv->layout, g_strdup (name), layout);
               gtk_container_add (GTK_CONTAINER (area), w);
 
-              *padded_layout = *layout;
+              *padded_layout = layout->rect;
 
               if (padded_layout->x + padded_layout->width <
                   area_rectangle->width)
@@ -1785,32 +1863,31 @@ hildon_home_area_remove (GtkContainer *area, GtkWidget *applet)
       g_signal_emit_by_name (area, "layout-changed");
     }
 
+  gtk_container_child_get (area, applet,
+                           "child-data", &child_data,
+                           NULL);
 
-      gtk_container_child_get (area, applet,
-                               "child-data", &child_data,
-                               NULL);
-
-      if (child_data)
-        {
+  if (child_data)
+    {
+      g_signal_handler_disconnect (applet,
+                                   child_data->realize_handler);
 
 #ifdef HAVE_X_COMPOSITE
-          if (HILDON_HOME_AREA_GET_CLASS (area)->composite)
-            {
-              g_signal_handler_disconnect (applet,
-                                           child_data->realize_handler);
-              g_signal_handler_disconnect (applet,
-                                           child_data->style_set_handler);
-              g_signal_handler_disconnect (applet,
-                                           child_data->size_allocate_handler);
-              g_signal_handler_disconnect (applet,
-                                           child_data->state_change_handler);
-            }
+      if (HILDON_HOME_AREA_GET_CLASS (area)->composite)
+        {
+          g_signal_handler_disconnect (applet,
+                                       child_data->style_set_handler);
+          g_signal_handler_disconnect (applet,
+                                       child_data->size_allocate_handler);
+          g_signal_handler_disconnect (applet,
+                                       child_data->state_change_handler);
+        }
 #endif
 
-          priv->children_data = g_list_remove (priv->children_data, child_data);
+      priv->children_data = g_list_remove (priv->children_data, child_data);
 
-          child_data_free (child_data);
-        }
+      child_data_free (child_data);
+    }
 }
 
 /* Public functions */
@@ -1855,41 +1932,40 @@ hildon_home_area_get_layout_mode (HildonHomeArea *area)
 }
 
 static void
-hildon_home_area_child_save_position (GtkWidget *widget, GKeyFile *keyfile)
+hildon_home_area_child_save_position (ChildData *child, GKeyFile *keyfile)
 {
   const gchar  *id;
-  gint          x, y;
 
-  if (!HILDON_DESKTOP_IS_HOME_ITEM (widget))
+  if (!HILDON_DESKTOP_IS_HOME_ITEM (child->widget))
     return;
 
-  id = hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (widget));
+  id = hildon_desktop_item_get_id (HILDON_DESKTOP_ITEM (child->widget));
   g_return_if_fail (id);
 
-  gtk_container_child_get (GTK_CONTAINER (widget->parent), widget,
-                           "x", &x,
-                           "y", &y,
-                           NULL);
-
   g_key_file_set_integer (keyfile,
                           id,
                           HH_APPLET_KEY_X,
-                          x);
+                          child->x);
 
   g_key_file_set_integer (keyfile,
                           id,
                           HH_APPLET_KEY_Y,
-                          y);
+                          child->y);
+
+  g_key_file_set_integer (keyfile,
+                          id,
+                          HH_APPLET_KEY_STACK_INDEX,
+                          child->stack_index);
 
   g_key_file_set_integer (keyfile,
                           id,
                           HH_APPLET_KEY_WIDTH,
-                          widget->allocation.width);
+                          child->widget->allocation.width);
 
   g_key_file_set_integer (keyfile,
                           id,
                           HH_APPLET_KEY_HEIGHT,
-                          widget->allocation.height);
+                          child->widget->allocation.height);
 
 }
 
@@ -1898,18 +1974,54 @@ hildon_home_area_save_configuration (HildonHomeArea *area,
                                      const gchar *path,
                                      GError **error)
 {
-  GKeyFile *keyfile;
-  FILE     *file;
-  gchar    *buffer = NULL;
-  guint     buffer_size;
-  GError   *local_error = NULL;
-  gint      ret;
+  HildonHomeAreaPriv   *priv;
+  GKeyFile             *keyfile;
+  FILE                 *file;
+  gchar                *buffer = NULL;
+  guint                 buffer_size;
+  GError               *local_error = NULL;
+  gint                  ret;
+  gint                  n_children, i;
+  Window               *wchildren, root, parent;
+
+  g_return_val_if_fail (HILDON_IS_HOME_AREA (area) && path, -1);
+
+  priv = HILDON_HOME_AREA_GET_PRIVATE (area);
+
+  gdk_error_trap_push ();
+  XQueryTree (GDK_DISPLAY (),
+              GDK_WINDOW_XID (GTK_WIDGET (area)->window),
+              &root,
+              &parent,
+              &wchildren,
+              &n_children);
+  if (gdk_error_trap_pop ())
+    {
+      if (wchildren)
+        XFree (wchildren);
+
+      return -1;
+    }
 
   keyfile = g_key_file_new ();
 
-  gtk_container_foreach (GTK_CONTAINER (area),
-                         (GtkCallback)hildon_home_area_child_save_position,
-                         keyfile);
+  for (i = 0; i < n_children; i++)
+    {
+      GList            *l = NULL;
+
+      l = g_list_find_custom (priv->children_data,
+                              &wchildren[i],
+                              (GCompareFunc)find_by_window);
+
+      if (l)
+        {
+          ChildData    *child = l->data;
+          child->stack_index = i;
+          hildon_home_area_child_save_position (child, keyfile);
+        }
+    }
+
+  XFree (wchildren);
 
   file = fopen (path, "w");
 
@@ -1997,7 +2109,7 @@ hildon_home_area_load_configuration (HildonHomeArea *area,
   while (n_groups > 0)
     {
       GtkWidget     *applet = NULL;
-      gint           x,y,width,height;
+      gint           x,y,width,height, stack_index;
       GList         *list_element;
 
       x = g_key_file_get_integer (keyfile,
@@ -2018,20 +2130,27 @@ hildon_home_area_load_configuration (HildonHomeArea *area,
                                        groups[n_groups-1],
                                        HH_APPLET_KEY_HEIGHT,
                                        &local_error);
+
+      stack_index = g_key_file_get_integer (keyfile,
+                                            groups[n_groups-1],
+                                            HH_APPLET_KEY_STACK_INDEX,
+                                            &local_error);
+
       if (local_error) goto cleanup;
 
       else
         {
-          GdkRectangle *rect = g_new (GdkRectangle, 1);
+          Layout *layout = g_new (Layout, 1);
 
-          rect->x = x;
-          rect->y = y;
-          rect->width = width;
-          rect->height = height;
+          layout->rect.x = x;
+          layout->rect.y = y;
+          layout->rect.width = width;
+          layout->rect.height = height;
+          layout->stack_index = stack_index;
 
           g_hash_table_insert (priv->layout,
                                g_strdup (groups[n_groups-1]),
-                               rect);
+                               layout);
         }
 
       /* Do we already have this applet loaded? */
@@ -2044,10 +2163,10 @@ hildon_home_area_load_configuration (HildonHomeArea *area,
       if (list_element)
         {
           applet = GTK_WIDGET (list_element->data);
-          gtk_fixed_move (GTK_FIXED (area),
-                          applet,
-                          x,
-                          y);
+          hildon_home_area_move (area,
+                                 applet,
+                                 x,
+                                 y);
 
           applets = g_list_remove (applets, applet);
         }
@@ -2132,7 +2251,10 @@ hildon_home_area_set_batch_add (HildonHomeArea *area, gboolean batch_add)
 }
 
 void
-hildon_home_area_put (HildonHomeArea *area, GtkWidget *widget, gint x, gint y)
+hildon_home_area_put (HildonHomeArea *area,
+                      GtkWidget *widget,
+                      gint x, gint y,
+                      gint stack_index)
 {
   ChildData    *child_data;
   gint          state = 0;
@@ -2150,20 +2272,28 @@ hildon_home_area_put (HildonHomeArea *area, GtkWidget *widget, gint x, gint y)
   child_data->widget = widget;
   child_data->x = x;
   child_data->y = y;
-#ifdef HAVE_X_COMPOSITE  
+  child_data->stack_index = stack_index;
+#ifdef HAVE_X_COMPOSITE
   child_data->state = state;
 #endif
   gtk_container_child_set (GTK_CONTAINER (area), widget,
                            "child-data", child_data,
                            NULL);
 
+  child_data->realize_handler =
+      g_signal_connect_after (widget, "realize",
+                              G_CALLBACK (hildon_home_area_child_realize),
+                              area);
+
+  if (GTK_WIDGET_REALIZED (widget))
+    {
+      hildon_home_area_sort_stack (area);
+      child_data->window = GDK_WINDOW_XID (widget->window);
+    }
+
 #ifdef HAVE_X_COMPOSITE
   if (HILDON_HOME_AREA_GET_CLASS (area)->composite)
     {
-      child_data->realize_handler =
-          g_signal_connect_after (widget, "realize",
-                                  G_CALLBACK (hildon_home_area_child_realize),
-                                  area);
       child_data->style_set_handler =
           g_signal_connect_after (widget, "style-set",
                                   G_CALLBACK (hildon_home_area_child_style_set),
@@ -2216,12 +2346,10 @@ hildon_home_area_move (HildonHomeArea *area, GtkWidget *widget, gint x, gint y)
                               widget->allocation.width,
                               widget->allocation.height};
 
-      gdk_window_invalidate_rect (widget->window, &invalid, TRUE);
-
+      gdk_window_invalidate_rect (GTK_WIDGET (area)->window, &invalid, TRUE);
       invalid.x = x + GTK_WIDGET (area)->allocation.x;
       invalid.y = y + GTK_WIDGET (area)->allocation.y;
-
-      gdk_window_invalidate_rect (widget->window, &invalid, TRUE);
+      gdk_window_invalidate_rect (GTK_WIDGET (area)->window, &invalid, TRUE);
     }
 
   data->x = x;
