@@ -1,9 +1,9 @@
 /* -*- mode:C; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
 
 /* 
- * This file is part of maemo-af-desktop
+ * This file is part of libhildonwm
  *
- * Copyright (C) 2005, 2006 Nokia Corporation.
+ * Copyright (C) 2005, 2006, 2007 Nokia Corporation.
  *
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  *
@@ -36,9 +36,10 @@
 #include <gtk/gtk.h>
 
 #include <libhildonwm/hd-wm-types.h>
-#include <libhildonwm/hd-entry-info.h>
-#include <libhildonwm/hd-wm-watchable-app.h>
-#include <libhildonwm/hd-wm-watched-window.h>
+#include <libhildonwm/hd-wm-entry-info.h>
+#include <libhildonwm/hd-wm-application.h>
+#include <libhildonwm/hd-wm-window.h>
+#include <libhildonwm/hd-wm-desktop.h>
 #include <libhildonwm/hd-keys.h>
 
 #define HN_WANT_DEBUG 0 /* Set to 1 for more verbose hn */
@@ -67,15 +68,15 @@
 
 /* DBus/Banner etc related defines
  */
-#define APP_LAUNCH_BANNER_MSG_LOADING        "ckct_ib_application_loading"
-#define APP_LAUNCH_BANNER_MSG_RESUMING       "ckct_ib_application_resuming"
-#define APP_LAUNCH_BANNER_MSG_LOADING_FAILED "ckct_ib_application_loading_failed"
+#define HDWM_APPLICATION_LAUNCH_BANNER_MSG_LOADING        "ckct_ib_application_loading"
+#define HDWM_APPLICATION_LAUNCH_BANNER_MSG_RESUMING       "ckct_ib_application_resuming"
+#define HDWM_APPLICATION_LAUNCH_BANNER_MSG_LOADING_FAILED "ckct_ib_application_loading_failed"
 /* Timeout of the launch banner, in secons */
-#define APP_LAUNCH_BANNER_TIMEOUT            20
+#define HDWM_APPLICATION_LAUNCH_BANNER_TIMEOUT            20
 /* Timeout of the launch banner in lowmem situation */
-#define APP_LAUNCH_BANNER_TIMEOUT_LOWMEM     40
+#define HDWM_APPLICATION_LAUNCH_BANNER_TIMEOUT_LOWMEM     40
 /* Interval for checking for new window or timeout, in seconds */
-#define APP_LAUNCH_BANNER_CHECK_INTERVAL     0.5
+#define HDWM_APPLICATION_LAUNCH_BANNER_CHECK_INTERVAL     0.5
 
 /* .desktop file related defines, mainly for keys
  */
@@ -114,6 +115,8 @@
 #define TN_DEFAULT_FOCUS FALSE
 #endif
 
+#define HD_WM_LAUNCHER_NAME "libhildonwm"
+
 #define HD_TYPE_WM            (hd_wm_get_type ())
 #define HD_WM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), HD_TYPE_WM, HDWM))
 #define HD_WM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  HD_TYPE_WM, HDWMClass))
@@ -138,13 +141,16 @@ struct _HDWMClass
 {
   GObjectClass parent_class;
 
-  void (*entry_info_added) 	   (HDWM *hdwm,HDEntryInfo *info);
-  void (*entry_info_removed) 	   (HDWM *hdwm,HDEntryInfo *info);
-  void (*entry_info_changed) 	   (HDWM *hdwm,HDEntryInfo *info);
-  void (*entry_info_stack_changed) (HDWM *hdwm,HDEntryInfo *info);
+  void (*entry_info_added) 	   (HDWM *hdwm,HDWMEntryInfo *info);
+  void (*entry_info_removed) 	   (HDWM *hdwm,HDWMEntryInfo *info);
+  void (*entry_info_changed) 	   (HDWM *hdwm,HDWMEntryInfo *info);
+  void (*entry_info_stack_changed) (HDWM *hdwm,HDWMEntryInfo *info);
   void (*work_area_changed)        (HDWM *hdwm,GdkRectangle *work_area);
   void (*show_menu)		   (HDWM *hdwm);
-  void (*application_starting)     (HDWM *hdwm, gchar *application);
+  void (*application_starting)     (HDWM *hdwm, gpointer application);
+  void (*application_died)	   (HDWM *hdwm, gpointer application);
+  void (*window_frozen)		   (HDWM *hdwm, HDWMWindow *window);
+  void (*window_frozen_cancel)     (HDWM *hdwm, HDWMWindow *window);
   /* */
 };
 
@@ -160,7 +166,7 @@ menuitem_comp_t;
 
 typedef struct 
 {
-  HDWMWatchedWindow* win;
+  HDWMWindow* win;
   GObject *button;
   gint vmdata;
   guint pid;
@@ -181,7 +187,7 @@ struct HDWMLaunchBannerInfo
   GtkWidget         *banner;
   struct timeval     launch_time;
   gchar             *msg;
-  HDWMWatchableApp  *app;
+  HDWMApplication  *app;
 };
 
 GType 
@@ -193,7 +199,7 @@ hd_wm_get_singleton (void);
 HDWM *
 hd_wm_get_singleton_without_dbus (void);
 
-HDEntryInfo *
+HDWMEntryInfo *
 hd_wm_get_home_info (HDWM *hdwm);
 
 /**  Send 'top' request for a certain existing window/view
@@ -202,7 +208,7 @@ hd_wm_get_home_info (HDWM *hdwm);
  *
  */
 void 
-hd_wm_top_item (HDEntryInfo *info);
+hd_wm_top_item (HDWMEntryInfo *info);
 
 
 /**  Send 'top' request for a certain service
@@ -223,24 +229,24 @@ hd_wm_top_desktop(void);
 void 
 hd_wm_toggle_desktop (void);
 
-HDWMWatchedWindow*
-hd_wm_lookup_watched_window_via_service (const gchar *service_name);
+HDWMWindow*
+hd_wm_lookup_window_via_service (const gchar *service_name);
 
-HDWMWatchedWindow*
-hd_wm_lookup_watched_window_via_menu_widget (GtkWidget *menu_widget);
+HDWMWindow*
+hd_wm_lookup_window_via_menu_widget (GtkWidget *menu_widget);
 
-HDWMWatchableApp*
-hd_wm_lookup_watchable_app_via_service (const gchar *service_name);
+HDWMApplication*
+hd_wm_lookup_application_via_service (const gchar *service_name);
 
-HDWMWatchableApp*
-hd_wm_lookup_watchable_app_via_exec (const gchar *exec_name);
+HDWMApplication*
+hd_wm_lookup_application_via_exec (const gchar *exec_name);
 
-HDWMWatchableApp*
-hd_wm_lookup_watchable_app_via_menu (GtkWidget *menu);
+HDWMApplication*
+hd_wm_lookup_application_via_menu (GtkWidget *menu);
 
 gchar *
-hd_wm_compute_watched_window_hibernation_key (Window xwin,
-					      HDWMWatchableApp *app);
+hd_wm_compute_window_hibernation_key (Window xwin,
+					      HDWMApplication *app);
 
 void
 hd_wm_monitor_register (void);
@@ -268,13 +274,13 @@ GList *
 hd_wm_get_applications (HDWM *hdwm);
 
 void 
-hd_wm_close_application (HDWM *hdwm, HDEntryInfo *entry_info);
+hd_wm_close_application (HDWM *hdwm, HDWMEntryInfo *entry_info);
 
-void 
-hd_wm_add_applications (HDWM *hdwm, HDEntryInfo *entry_info);
+/*void 
+hd_wm_add_applications (HDWM *hdwm, HDEntryInfo *entry_info);*/
 
 gboolean  
-hd_wm_remove_applications (HDWM *hdwm, HDEntryInfo *entry_info);
+hd_wm_remove_applications (HDWM *hdwm, HDWMEntryInfo *entry_info);
 
 void 
 hd_wm_update_client_list (HDWM *hdwm);
@@ -284,14 +290,17 @@ hd_wm_activate_service (const gchar *service_name,
                         const gchar *launch_parameters);
 
 void 
-hd_wm_switch_application_window (HDWM *hdwm, HDWMWatchableApp *app, gboolean to_next);
+hd_wm_switch_application_window (HDWM *hdwm, HDWMApplication *app, gboolean to_next);
 
 void 
-hd_wm_switch_info_window (HDWM *hdwm, HDEntryInfo *entry_info, gboolean to_next);
+hd_wm_switch_info_window (HDWM *hdwm, HDWMEntryInfo *entry_info, gboolean to_next);
 
 void 
 hd_wm_switch_instance_current_window (HDWM *hdwm, gboolean to_next);
 
+GHashTable *
+hd_wm_get_icon_cache (HDWM *hdwm);
+
 /*
  * These are simple getters/setters that replace direct use of global
  * hnwm->something. In order to ensure that we do not incure performance
@@ -303,7 +312,7 @@ extern inline Atom
 hd_wm_get_atom(gint indx);
 
 extern inline GHashTable *
-hd_wm_get_watched_windows(void);
+hd_wm_get_windows(void);
 
 extern inline GHashTable *
 hd_wm_get_hibernating_windows(void);
@@ -344,10 +353,10 @@ hd_wm_get_lowmem_banner_timeout(void);
 extern inline gulong
 hd_wm_get_lowmem_timeout_multiplier(void);
 
-extern inline HDWMWatchedWindow *
+extern inline HDWMWindow *
 hd_wm_get_active_window(void);
 
-extern inline HDWMWatchedWindow *
+extern inline HDWMWindow *
 hd_wm_get_last_active_window(void);
 
 extern inline gboolean
@@ -358,7 +367,7 @@ hd_wm_modal_windows_present(void);
  *
  * NB: we intentionally do not provide a setter -- the active window is our
  * business alone and must not be done from anywhere else than the WM itself.
- * The reset function is only to be called from hd_wm_watched_window_destroy().
+ * The reset function is only to be called from hd_wm_window_destroy().
  */
 extern inline void
 hd_wm_reset_active_window(void);
