@@ -62,13 +62,20 @@ G_DEFINE_TYPE (HDDesktop, hd_desktop, G_TYPE_OBJECT);
 #define HD_DESKTOP_CONFIG_FILE         "desktop.conf"
 #define HD_DESKTOP_CONFIG_USER_PATH    ".osso/hildon-desktop/"
 
+#define HD_DESKTOP_PING_TIMEOUT_MESSAGE_STRING       _( "qgn_nc_apkil_notresponding" )
+#define HD_DESKTOP_PING_TIMEOUT_RESPONSE_STRING      _( "qgn_ib_apkil_responded" )
+#define HD_DESKTOP_PING_TIMEOUT_KILL_FAILURE_STRING  _( "" )
+
+#define HD_DESKTOP_PING_TIMEOUT_BUTTON_OK_STRING     _( "qgn_bd_apkil_ok" )
+#define HD_DESKTOP_PING_TIMEOUT_BUTTON_CANCEL_STRING _( "qgn_bd_apkil_cancel" )
+
 typedef struct 
 {
   GtkWidget         *parent;
   GtkWidget         *banner;
   struct timeval     launch_time;
   gchar             *msg;
-  HDWMWatchableApp  *app;
+  HDWMApplication  *app;
   HDWM 		    *hdwm;
 } HDDesktopBannerInfo;
 
@@ -80,6 +87,7 @@ typedef struct
   HDDesktop              *desktop;
   HDUIPolicy             *policy;
   gboolean                is_ordered;
+  gboolean                load_new_plugins;
   GnomeVFSMonitorHandle  *plugin_dir_monitor;
 } HDDesktopContainerInfo;
 
@@ -112,8 +120,99 @@ hd_desktop_launch_banner_timeout (gpointer data);
 void 
 hd_desktop_launch_banner_close (GtkWidget *parent, HDDesktopBannerInfo *info);
 
+static void
+hd_desktop_ping_timeout_dialog_response (GtkDialog *note, gint ret, gpointer data)
+{
+  HDWMWindow *win = (HDWMWindow *)data;
+  HDWMApplication *app = hd_wm_window_get_application (win);
+
+  gtk_widget_destroy (GTK_WIDGET(note));
+  hd_wm_application_set_ping_timeout_note (app, NULL);
+
+  if (ret == GTK_RESPONSE_OK)
+  {
+    /* Kill the app */
+    if (!hd_wm_window_attempt_signal_kill (win, SIGKILL, FALSE))
+      g_debug ("hd_wm_ping_timeout:failed to kill application '%s'.", hd_wm_window_get_name (win));
+  }
+}
+
+static void
+hd_desktop_application_frozen (HDWM *hdwm, HDWMWindow *win, gpointer data)
+{
+  GtkWidget *note;
+
+  HDWMApplication *app = hd_wm_window_get_application (win);
+
+  gchar *timeout_message = 
+    g_strdup_printf (HD_DESKTOP_PING_TIMEOUT_MESSAGE_STRING, hd_wm_window_get_name (win));
+
+  /* FIXME: Do we need to check if the note already exists? */
+  note = GTK_WIDGET (hd_wm_application_get_ping_timeout_note (app));
+  
+  if (note && GTK_IS_WIDGET (note))
+  {
+    g_debug ("hd_wm_ping_timeout: the note already exists.");
+    goto cleanup_and_exit;
+  }
+
+  note = hildon_note_new_confirmation (NULL, timeout_message);
+
+  hd_wm_application_set_ping_timeout_note (app, G_OBJECT (note));
+
+  hildon_note_set_button_texts (HILDON_NOTE (note),
+                                HD_DESKTOP_PING_TIMEOUT_BUTTON_OK_STRING,
+                                HD_DESKTOP_PING_TIMEOUT_BUTTON_CANCEL_STRING);
+
+  g_signal_connect (G_OBJECT (note),
+                    "response",
+                    G_CALLBACK (hd_desktop_ping_timeout_dialog_response),
+                    win);
+
+  gtk_widget_show_all (note);
+
+cleanup_and_exit:
+
+  g_free (timeout_message);
+}
+
+
+static void
+hd_desktop_application_frozen_cancel (HDWM *hdwm, HDWMWindow *win, gpointer data)
+{
+  HDWMApplication *app = hd_wm_window_get_application (win);
+
+  GObject *note = hd_wm_application_get_ping_timeout_note (app);
+
+  gchar *response_message = 
+    g_strdup_printf (HD_DESKTOP_PING_TIMEOUT_RESPONSE_STRING, hd_wm_window_get_name (win));
+
+  if (note && GTK_IS_WIDGET (note))
+    gtk_dialog_response (GTK_DIALOG (note), GTK_RESPONSE_CANCEL);
+
+  /* Show the infoprint */
+  hildon_banner_show_information (NULL, NULL, response_message);
+
+  g_free (response_message);
+}
+
+static void
+hd_desktop_application_died_dialog (HDWM *hdwm, gpointer text, gpointer data)
+{
+  gchar *_text = (gchar *) text;	
+  GtkWidget *dialog;
+	
+  dialog = hildon_note_new_information (NULL, text);
+  
+  gtk_widget_show_all(dialog);
+  gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_widget_destroy(dialog);
+  
+  g_free (_text);
+}
+
 static void 
-hd_desktop_launch_banner_show (HDWM *hdwm, HDWMWatchableApp *app, gpointer data)
+hd_desktop_launch_banner_show (HDWM *hdwm, gpointer app, gpointer data)
 {
   HDDesktopBannerInfo *info;
   guint                interval;
@@ -121,22 +220,28 @@ hd_desktop_launch_banner_show (HDWM *hdwm, HDWMWatchableApp *app, gpointer data)
 
   g_return_if_fail (app);
 
-  interval = APP_LAUNCH_BANNER_CHECK_INTERVAL * 1000;
+  interval = HDWM_APPLICATION_LAUNCH_BANNER_CHECK_INTERVAL * 1000;
 
   info = g_new0 (HDDesktopBannerInfo, 1);
 
   info->hdwm = hdwm;
-  info->app  = app;
+  info->app  = (HD_WM_IS_APPLICATION (app)) ? app : NULL;
 
   gettimeofday (&info->launch_time, NULL );
 
-  lapp_name = hd_wm_watchable_app_get_localized_name (app);
+  lapp_name = (HD_WM_IS_APPLICATION (app)) ? 
+	      hd_wm_application_get_localized_name (app) : app;
 
-  info->msg = 
-    g_strdup_printf (_(hd_wm_watchable_app_is_hibernating (app) ?
-                     APP_LAUNCH_BANNER_MSG_RESUMING :
-                     APP_LAUNCH_BANNER_MSG_LOADING),
-                     lapp_name ? _(lapp_name) : "" );
+  if (HD_WM_IS_APPLICATION (app))
+    info->msg = 
+      g_strdup_printf (_(hd_wm_application_is_hibernating (app) ?
+                       HDWM_APPLICATION_LAUNCH_BANNER_MSG_RESUMING :
+                       HDWM_APPLICATION_LAUNCH_BANNER_MSG_LOADING),
+                       lapp_name ? _(lapp_name) : "" );
+  else
+    info->msg =
+      g_strdup_printf (HDWM_APPLICATION_LAUNCH_BANNER_MSG_LOADING,
+	  	       lapp_name ? _(lapp_name) : "" );	      
 
   info->banner = GTK_WIDGET (hildon_banner_show_animation (NULL, NULL, info->msg));
 
@@ -163,7 +268,7 @@ hd_desktop_launch_banner_timeout (gpointer data)
   time_left = (guint) (t2 - t1);
 
   if (time_left >= current_banner_timeout + 4)/* ||
-      hd_wm_watchable_app_has_windows (info->app))*/
+      hd_wm_application_has_windows (info->app))*/
   {
     hd_desktop_launch_banner_close (NULL, info);
     return FALSE;
@@ -604,9 +709,20 @@ hd_desktop_plugin_dir_changed (GnomeVFSMonitorHandle *handle,
       event_type != GNOME_VFS_MONITOR_EVENT_CREATED)
     return;
 
-  if (event_type == GNOME_VFS_MONITOR_EVENT_CREATED)
+  if (info->load_new_plugins &&
+      event_type == GNOME_VFS_MONITOR_EVENT_CREATED)
+  {
+    GnomeVFSURI *uri = gnome_vfs_uri_new (info_uri);
+    gchar *uri_str;
+    
     update = TRUE;
-  
+
+    uri_str = 
+      gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+
+    plugin_list = g_list_append (plugin_list, uri_str);
+  }
+    
   children = gtk_container_get_children (
                   HILDON_DESKTOP_WINDOW (info->container)->container);
 
@@ -698,7 +814,7 @@ hd_desktop_load_containers (HDDesktop *desktop)
     GList *plugin_list = NULL;
     gchar *type, *container_config, *container_config_file, *plugin_dir;
     gchar *policy_module;
-    gboolean is_ordered;
+    gboolean is_ordered, load_new_plugins;
     
     error = NULL;
 
@@ -714,6 +830,18 @@ hd_desktop_load_containers (HDDesktop *desktop)
       error = NULL;
     }
     
+    load_new_plugins = g_key_file_get_boolean (keyfile, 
+                                               groups[i], 
+                                               HD_DESKTOP_CONFIG_KEY_LOAD_NEW_PLUGINS,
+                                               &error);
+
+    if (error)
+    {
+      load_new_plugins = FALSE;
+      g_error_free (error);
+      error = NULL;
+    }
+    
     policy_module = g_key_file_get_string (keyfile, 
                                            groups[i], 
                                            HD_DESKTOP_CONFIG_KEY_UI_POLICY,
@@ -1042,6 +1170,7 @@ hd_desktop_load_containers (HDDesktop *desktop)
     info->desktop = desktop;
     info->policy = policy;
     info->is_ordered = is_ordered;
+    info->load_new_plugins = load_new_plugins;
  
     g_signal_connect (G_OBJECT (info->container), 
                       "select-plugins",
@@ -1435,6 +1564,21 @@ hd_desktop_init (HDDesktop *desktop)
 		    G_CALLBACK (hd_desktop_launch_banner_show),
 		    NULL);
 
+  g_signal_connect (hdwm,
+		    "application-died",
+		    G_CALLBACK (hd_desktop_application_died_dialog),
+		    NULL);
+  
+  g_signal_connect (hdwm,
+		    "application-frozen",
+		    G_CALLBACK (hd_desktop_application_frozen),
+		    NULL);
+
+  g_signal_connect (hdwm,
+		    "application-frozen-cancel",
+		    G_CALLBACK (hd_desktop_application_frozen_cancel),
+		    NULL);
+  
   desktop->priv->system_conf_monitor = NULL;
   desktop->priv->user_conf_monitor = NULL;
 
