@@ -39,11 +39,15 @@
 
 #include <libhildondesktop/hildon-home-area.h>
 #include <libhildondesktop/hildon-desktop-home-item.h>
+#include <libhildondesktop/hildon-desktop-picture.h>
 
 #include <gtk/gtkcheckmenuitem.h>
 #include <gtk/gtkspinbutton.h>
 
+#include <dbus/dbus-glib.h>
+
 #include <X11/Xatom.h>
+#include <X11/extensions/Xrender.h>
 
 #ifdef HAVE_LIBHILDON
 #include <hildon/hildon-banner.h>
@@ -106,12 +110,11 @@ struct _HDHomeWindowPrivate
   GtkWidget        *layout_mode_banner;
   gint              layout_mode_banner_to;
 
-  gchar            *north_border;
-  gchar            *west_border;
-
   HDHomeBackground *background;
   HDHomeBackground *previous_background;
 
+  Picture           background_picture;
+
   gboolean          screen_is_off;
 
   gboolean          selecting_applets;
@@ -121,11 +124,10 @@ struct _HDHomeWindowPrivate
   GtkWidget        *cancel_note;
   guint             background_loading_timeout;
   guint             cancel_note_update_timeout;
-  gboolean          background_cancelled;
 };
 
 /* Properties */
-enum 
+enum
 {
   PROP_0,
   PROP_OSSO_CONTEXT,
@@ -150,11 +152,6 @@ hd_home_window_set_property (GObject      *gobject,
                              const GValue *value,
                              GParamSpec   *pspec);
 
-static void
-hd_home_window_notify (GObject     *object,
-                       GParamSpec  *pspec);
-
-
 #ifdef HAVE_LIBOSSO
 static void
 hd_home_window_set_osso_context (HDHomeWindow *window, osso_context_t *osso);
@@ -226,6 +223,10 @@ hd_home_window_map_event (GtkWidget    *widget,
 static void
 hd_home_window_realize (GtkWidget *widget);
 
+static gboolean
+hd_home_window_expose (GtkWidget       *widget,
+                       GdkEventExpose  *event);
+
 static void
 hd_home_window_unrealize (GtkWidget *widget);
 
@@ -286,7 +287,8 @@ static void
 hd_home_window_cancel_note_hide (HDHomeWindow *window);
 
 static void
-hd_home_window_cancel_note_show (HDHomeWindow *window);
+hd_home_window_cancel_note_show (HDHomeWindow *window,
+                                 HDHomeBackground *background);
 
 #if 0
 static void
@@ -308,13 +310,13 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   object_class->constructor  = hd_home_window_constructor;
   object_class->set_property = hd_home_window_set_property;
   object_class->get_property = hd_home_window_get_property;
-  object_class->notify       = hd_home_window_notify;
 
   widget_class->map_event = hd_home_window_map_event;
   widget_class->style_set = hd_home_window_style_set;
   widget_class->key_press_event = hd_home_window_key_press_event;
   widget_class->realize   = hd_home_window_realize;
   widget_class->unrealize = hd_home_window_unrealize;
+  widget_class->expose_event = hd_home_window_expose;
 
   hhwindow_class->button1_clicked = hd_home_window_button1_clicked;
   hhwindow_class->button2_clicked = hd_home_window_button2_clicked;
@@ -628,24 +630,9 @@ hd_home_window_get_property (GObject    *gobject,
   }
 }
 
-static void
-hd_home_window_notify (GObject     *object,
-                       GParamSpec  *pspec)
-{
-  HDHomeWindowPrivate  *priv;
-  priv = HD_HOME_WINDOW_GET_PRIVATE (object);
-
-  if (g_str_equal (pspec->name, "work-area"))
-    {
-      g_debug ("Work area changed, reapplying the background");
-      hd_home_window_set_background (HD_HOME_WINDOW (object),
-                                     priv->background);
-    }
-}
-
 static void
 background_apply_callback (HDHomeBackground *background,
-                           gint              pixmap_xid,
+                           Picture           picture,
                            GError           *error,
                            HDHomeWindow     *window)
 {
@@ -660,24 +647,48 @@ background_apply_callback (HDHomeBackground *background,
       priv->background_loading_timeout = 0;
     }
 
-  if (priv->background_cancelled)
-    {
-      priv->background_cancelled = FALSE;
-      return;
-    }
-
   if (GTK_IS_WIDGET (priv->cancel_note))
     hd_home_window_cancel_note_hide (window);
 
   if (error)
     {
-      g_warning ("Got error when apply background: %s",
-                 error->message);
+      const gchar *text = NULL;
+      g_warning ("Got error when apply background: %i %i %s  %s",
+                 error->domain,
+                 error->code,
+                 error->message,
+      dbus_g_error_get_name (error));
+
+      if (dbus_g_error_has_name (error,
+                                 "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code2"))
+        text = HH_FILE_CORRUPTED_TEXT;
+      else if ((dbus_g_error_has_name (error,
+                                       "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code1")))
+        text = HH_NO_CONNECTION_TEXT;
+      else if ((dbus_g_error_has_name (error,
+                                       "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code4")))
+        text = HH_MMC_OPEN_TEXT;
+      else if ((dbus_g_error_has_name (error,
+                                       "org.freedesktop.DBus.GLib.UnmappedError.BackgroundManagerErrorQuark.Code7")))
+        text = HH_NO_CONNECTION_TEXT;
+
+      if (text)
+        hd_home_window_show_information_note (window, text);
+
       return;
     }
 
-  gdk_window_clear (GTK_WIDGET (window)->window);
-  gtk_widget_queue_draw (GTK_WIDGET (window));
+  if (priv->background_picture)
+    XRenderFreePicture (GDK_DISPLAY (), priv->background_picture);
+
+  priv->background_picture = picture;
+  g_debug ("Received picture %i", (gint)priv->background_picture);
+
+  if (GTK_WIDGET_VISIBLE (GTK_WIDGET (window)))
+    gdk_window_invalidate_rect (GTK_WIDGET (window)->window,
+                                (GdkRectangle*)(&GTK_WIDGET (window)->allocation),
+                                TRUE);
+
 
   if (background != priv->background)
     {
@@ -690,23 +701,20 @@ background_apply_callback (HDHomeBackground *background,
 
 static void
 background_apply_and_save_callback (HDHomeBackground *background,
-                                    gint              pixmap_xid,
+                                    Picture           picture,
                                     GError           *error,
                                     HDHomeWindow     *window)
 {
   HDHomeWindowPrivate  *priv;
   gchar                *conffile;
   GError               *save_error = NULL;
-  gboolean              background_cancelled;
 
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
 
-  background_cancelled = priv->background_cancelled;
-
-  background_apply_callback (background, pixmap_xid, error, window);
+  background_apply_callback (background, picture, error, window);
 
   /* Do not save if an error occurred */
-  if (error || background_cancelled)
+  if (error)
     return;
 
   conffile = g_build_path (G_DIR_SEPARATOR_S,
@@ -723,7 +731,7 @@ background_apply_and_save_callback (HDHomeBackground *background,
     {
       g_warning ("Error when saving background settings to %s: %s",
                  conffile,
-                 error->message);
+                 save_error->message);
       g_error_free (save_error);
     }
 
@@ -812,6 +820,48 @@ hd_home_window_root_event_filter (GdkXEvent *xevent,
   return GDK_FILTER_CONTINUE;
 }
 
+static gboolean
+hd_home_window_expose (GtkWidget *widget, GdkEventExpose *event)
+{
+  if (GTK_WIDGET_DRAWABLE (widget))
+    {
+      HDHomeWindowPrivate      *priv;
+
+      priv = HD_HOME_WINDOW_GET_PRIVATE (widget);
+
+      if (priv->background_picture != None)
+        {
+          Picture                   picture;
+          GdkDrawable              *drawable;
+          gint                      x_offset, y_offset;
+
+          gdk_window_get_internal_paint_info (widget->window,
+                                              &drawable,
+                                              &x_offset, &y_offset);
+
+          picture = hildon_desktop_picture_from_drawable (drawable);
+
+          XRenderComposite (GDK_DISPLAY (),
+                            PictOpSrc,
+                            priv->background_picture,
+                            None,
+                            picture,
+                            0, 0,
+                            0, 0,
+                            - x_offset, - y_offset,
+                            widget->allocation.width, widget->allocation.height);
+
+          XRenderFreePicture (GDK_DISPLAY (), picture);
+          XSync (GDK_DISPLAY (), FALSE);
+        }
+
+      return GTK_WIDGET_CLASS (hd_home_window_parent_class)->
+          expose_event (widget, event);
+    }
+
+  return FALSE;
+}
+
 static void
 hd_home_window_realize (GtkWidget *widget)
 {
@@ -1380,7 +1430,6 @@ hd_home_window_area_changed (HDHomeWindow *window)
   HDHomeWindowPrivate  *priv;
 
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
-  g_debug ("layout changed, saving");
   if (!priv->save_area_timeout)
     priv->save_area_timeout = g_timeout_add (SAVE_TIMEOUT,
                                              (GSourceFunc)
@@ -1468,8 +1517,6 @@ static void
 hd_home_window_osso_display_state_cb (osso_display_state_t      state,
                                       gpointer                  window)
 {
-  g_return_if_fail (state);
-
   g_signal_emit_by_name (G_OBJECT (window),
                          "screen-off",
                          state == OSSO_DISPLAY_OFF);
@@ -1733,14 +1780,21 @@ hd_home_window_accept_layout (HDHomeWindow *window)
   g_signal_emit_by_name (window, "layout-mode-accept");
 }
 
-static gboolean
-hd_home_window_set_background_timeout (HDHomeWindow *window)
+typedef struct
 {
-  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  HDHomeWindow         *window;
+  HDHomeBackground     *background;
+} BackgroundTimeoutData;
+
+static gboolean
+hd_home_window_set_background_timeout (BackgroundTimeoutData *data)
+{
+  HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (data->window);
 
   priv->background_loading_timeout = 0;
 
-  hd_home_window_cancel_note_show (window);
+  hd_home_window_cancel_note_show (data->window, data->background);
+  g_free (data);
   return FALSE;
 }
 
@@ -1765,13 +1819,16 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
           if (!hd_home_background_equal (priv->background,
                                          background))
             {
+              BackgroundTimeoutData *data = g_new (BackgroundTimeoutData, 1);
+              data->window = window;
+              data->background = background;
               priv->background_loading_timeout =
                   g_timeout_add (1000,
                                  (GSourceFunc)
                                    hd_home_window_set_background_timeout,
-                                 window);
+                                 data);
               hd_home_background_apply_async
-                  (hd_home_background_copy (background),
+                  (background,
                    GTK_WIDGET (window)->window,
                    workarea,
                    (HDHomeBackgroundApplyCallback)background_apply_and_save_callback,
@@ -1781,13 +1838,17 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
           break;
       case HILDON_HOME_SET_BG_RESPONSE_PREVIEW:
             {
+              BackgroundTimeoutData *data = g_new (BackgroundTimeoutData, 1);
+              data->window = window;
+              data->background = background;
+
               priv->background_loading_timeout =
                   g_timeout_add (1000,
                                  (GSourceFunc)
                                    hd_home_window_set_background_timeout,
-                                 window);
+                                 data);
               hd_home_background_apply_async
-                  (hd_home_background_copy (background),
+                   (background,
                    GTK_WIDGET (window)->window,
                    workarea,
                    (HDHomeBackgroundApplyCallback)background_apply_callback,
@@ -1800,11 +1861,6 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
           if (!hd_home_background_equal (priv->background,
                                          priv->previous_background))
             {
-              priv->background_loading_timeout =
-                  g_timeout_add (1000,
-                                 (GSourceFunc)
-                                   hd_home_window_set_background_timeout,
-                                 window);
               hd_home_background_apply_async
                   (priv->previous_background,
                    GTK_WIDGET (window)->window,
@@ -1907,10 +1963,15 @@ static void
 hd_home_window_cancel_note_response (HDHomeWindow *window)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  HDHomeBackground     *background;
+
+  background = g_object_get_data (G_OBJECT (priv->cancel_note), "background");
+
+  if (HD_IS_HOME_BACKGROUND (background))
+    hd_home_background_cancel (background);
 
   hd_home_window_cancel_note_hide (window);
 
-  priv->background_cancelled = TRUE;
 }
 
 static gboolean
@@ -1924,7 +1985,8 @@ hd_home_window_cancel_note_update (GtkWidget *prog_bar)
 }
 
 static void
-hd_home_window_cancel_note_show (HDHomeWindow *window)
+hd_home_window_cancel_note_show (HDHomeWindow          *window,
+                                 HDHomeBackground      *background)
 {
   HDHomeWindowPrivate  *priv = HD_HOME_WINDOW_GET_PRIVATE (window);
   GtkWidget            *label, *prog_bar;
@@ -1932,6 +1994,7 @@ hd_home_window_cancel_note_show (HDHomeWindow *window)
   if (priv->cancel_note)
     return;
 
+
   priv->cancel_note =
       gtk_dialog_new_with_buttons ("",
                                    NULL,
@@ -1940,6 +2003,9 @@ hd_home_window_cancel_note_show (HDHomeWindow *window)
                                    HH_SET_BG_CANCEL, GTK_RESPONSE_CANCEL,
                                    NULL);
 
+  g_object_set_data (G_OBJECT (priv->cancel_note),
+                     "background", background);
+
   label = gtk_label_new (HH_CANCEL_BG_TEXT);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (priv->cancel_note)->vbox),
                      label);
